varying vec2 texc;
uniform vec2 ver_mouth;
uniform vec2 hor_mouth;
uniform vec4 offset_wh;

uniform float intensity_mouthsize;
uniform float scale_mouthsize;
uniform sampler2D tex_mouthsize;

uniform float intensity_mouthwidth;
uniform float left_intensity_mouthwidth;
uniform float right_intensity_mouthwidth;
uniform float left_scale_mouthwidth;
uniform float right_scale_mouthwidth;
uniform sampler2D tex_mouthwidth;

uniform float intensity_mouththickness;
uniform float up_intensity_mouththickness;
uniform float down_intensity_mouththickness;
uniform float scale_mouththickness;
uniform sampler2D tex_mouththickness_up;
uniform sampler2D tex_mouththickness_down;

uniform float intensity_mouthsmile;
uniform float scale_mouthsmile;
uniform sampler2D tex_mouthsmile;

uniform float intensity_mouthmshape;
uniform float scale_mouthmshape;
uniform sampler2D tex_mouthmshape;

uniform float intensity_mouthheight;
uniform float scale_mouthheight;
uniform float intensity_mouthslant;
uniform float scale_mouthslant;
uniform mat4 M_mouthslant;
uniform sampler2D tex_mouthmask;

uniform sampler2D tex_uv_in;

vec2 MouthSize(vec2 texc_warp) {
    vec2 out_texc = texc_warp;
    vec4 maskColor = texture2D(tex_mouthsize, texc + (out_texc - texc) / offset_wh.zw);
    vec2 newCoords = vec2(0.0);
    newCoords = (maskColor.rg - 127.0 / 255.0) / 25.0;

    vec2 off = abs(vec2(1.0) - texc_warp);
    vec2 off1 = abs(texc_warp);
    vec2 min_off = min(off, off1);
    vec2 modify_intensity = clamp(smoothstep(vec2(0.0), vec2(0.15, 0.1), min_off), vec2(0.55, 0.0), vec2(1.0, 1.0));

    vec2 newCoord_x = hor_mouth * newCoords.x;
    vec2 newCoord_y = ver_mouth * newCoords.y;

    if(abs(intensity_mouthsize) > 0.001){
         out_texc += (newCoord_x + newCoord_y) * maskColor.a * intensity_mouthsize * modify_intensity * scale_mouthsize;
    }
    
    return out_texc;
}

vec2 MouthWidth(vec2 texc_warp) {
    vec2 out_texc = texc_warp;
    vec2 off = abs(vec2(1.0) - texc_warp);
    vec2 off1 = abs(texc_warp);
    vec2 min_off = min(off, off1);
    vec2 modify_intensity = clamp(smoothstep(vec2(0.0), vec2(0.15, 0.1), min_off), vec2(0.55, 0.0), vec2(1.0, 1.0));

    vec4 maskColor = texture2D(tex_mouthwidth, texc + (out_texc - texc) / offset_wh.zw);
    vec2 newCoords = (maskColor.rg - 127.0 / 255.0) / 25.0;
    vec2 newCoord_x = hor_mouth * newCoords.x;
    vec2 newCoord_y = ver_mouth * newCoords.y; 
    if(abs(left_intensity_mouthwidth) > 0.001){      
        if (maskColor.b < 0.5) {
            out_texc += (newCoord_x + newCoord_y) * maskColor.a * left_intensity_mouthwidth * modify_intensity * left_scale_mouthwidth * smoothstep(0.5,0.48,maskColor.b);
        } 
    }

    if(abs(right_intensity_mouthwidth) > 0.001){
        if (maskColor.b > 0.5) {
           out_texc += (newCoord_x + newCoord_y) * maskColor.a * right_intensity_mouthwidth * modify_intensity * right_scale_mouthwidth * smoothstep(0.5,0.52,maskColor.b);
        }
    }

    maskColor = texture2D(tex_mouthwidth, texc + (out_texc - texc) / offset_wh.zw);
    newCoords = (maskColor.rg - 127.0 / 255.0) / 25.0;
    newCoord_x = hor_mouth * newCoords.x;
    newCoord_y = ver_mouth * newCoords.y; 
    if(abs(intensity_mouthwidth) > 0.001){
        if (maskColor.b < 0.5) {
            out_texc += (newCoord_x + newCoord_y) * maskColor.a * intensity_mouthwidth * modify_intensity * left_scale_mouthwidth;
        } else {
            out_texc += (newCoord_x + newCoord_y) * maskColor.a * intensity_mouthwidth * modify_intensity * right_scale_mouthwidth;
        }
    }
    
    return out_texc;
}

vec2 MouthThickness(vec2 texc_warp) {
    vec2 out_texc = texc_warp;
    vec2 off = abs(vec2(1.0) - texc_warp);
    vec2 off1 = abs(texc_warp);
    vec2 min_off = min(off, off1);
    vec2 modify_intensity = clamp(smoothstep(vec2(0.0), vec2(0.15, 0.1), min_off), vec2(0.55, 0.0), vec2(1.0, 1.0));

    vec4 maskColor_up = texture2D(tex_mouththickness_up, texc + (out_texc - texc) / offset_wh.zw);
    vec2 newCoords_up = (maskColor_up.rg - 127.0 / 255.0) / 25.0;
    vec2 newCoord_up_x = hor_mouth * newCoords_up.x;
    vec2 newCoord_up_y = ver_mouth * newCoords_up.y;    
    if(abs(up_intensity_mouththickness) > 0.001){  
        out_texc += (newCoord_up_x + newCoord_up_y) * maskColor_up.a * up_intensity_mouththickness * modify_intensity * scale_mouththickness;
    }

    vec4 maskColor_down = texture2D(tex_mouththickness_down, texc + (out_texc - texc) / offset_wh.zw);
    vec2 newCoords_down = (maskColor_down.rg - 127.0 / 255.0) / 25.0;
    vec2 newCoord_down_x = hor_mouth * newCoords_down.x;
    vec2 newCoord_down_y = ver_mouth * newCoords_down.y;
    if(abs(down_intensity_mouththickness) > 0.001){     
        float down_neg_rate =  down_intensity_mouththickness < 0.0 ? 1.25 : 1.0;
        out_texc += (newCoord_down_x + newCoord_down_y) * maskColor_down.a * down_intensity_mouththickness * modify_intensity * scale_mouththickness * down_neg_rate;
    }

    maskColor_up = texture2D(tex_mouththickness_up, texc + (out_texc - texc) / offset_wh.zw);
    maskColor_down = texture2D(tex_mouththickness_down, texc + (out_texc - texc) / offset_wh.zw);
    newCoords_up = (maskColor_up.rg - 127.0 / 255.0) / 25.0;
    newCoords_down = (maskColor_down.rg - 127.0 / 255.0) / 25.0;
    newCoord_up_x = hor_mouth * newCoords_up.x;
    newCoord_up_y = ver_mouth * newCoords_up.y;
    newCoord_down_x = hor_mouth * newCoords_down.x;
    newCoord_down_y = ver_mouth * newCoords_down.y;
    if(abs(intensity_mouththickness) > 0.001){
        out_texc += (newCoord_up_x + newCoord_up_y) * maskColor_up.a * intensity_mouththickness * modify_intensity * scale_mouththickness;
        float down_neg_rate =  intensity_mouththickness < 0.0 ? 1.25 : 1.0;
        out_texc += (newCoord_down_x + newCoord_down_y) * maskColor_down.a * intensity_mouththickness * modify_intensity * scale_mouththickness * down_neg_rate;
    }
    
    return out_texc;
}

vec2 MouthSmile(vec2 texc_warp) {
    vec2 out_texc = texc_warp;
    vec4 maskColor = texture2D(tex_mouthsmile, texc + (out_texc - texc) / offset_wh.zw);
    vec2 newCoords = vec2(0.0);
    newCoords = (maskColor.rg - 127.0 / 255.0) / 25.0;

    vec2 off = abs(vec2(1.0) - texc_warp);
    vec2 off1 = abs(texc_warp);
    vec2 min_off = min(off, off1);
    vec2 modify_intensity = clamp(smoothstep(vec2(0.0), vec2(0.15, 0.1), min_off), vec2(0.55, 0.0), vec2(1.0, 1.0));

    vec2 newCoord_x = hor_mouth * newCoords.x;
    vec2 newCoord_y = ver_mouth * newCoords.y;

    if(abs(intensity_mouthsmile) > 0.001){
        out_texc += (newCoord_x + newCoord_y) * maskColor.a * intensity_mouthsmile * modify_intensity * scale_mouthsmile;
    }
    
    return out_texc;
}

vec2 MouthMshape(vec2 texc_warp) {
    vec2 out_texc = texc_warp;
    vec4 maskColor = texture2D(tex_mouthmshape, texc + (out_texc - texc) / offset_wh.zw);
    vec2 newCoords = vec2(0.0);
    newCoords = (maskColor.rg - 127.0 / 255.0) / 25.0;

    vec2 off = abs(vec2(1.0) - texc_warp);
    vec2 off1 = abs(texc_warp);
    vec2 min_off = min(off, off1);
    vec2 modify_intensity = clamp(smoothstep(vec2(0.0), vec2(0.15, 0.1), min_off), vec2(0.55, 0.0), vec2(1.0, 1.0));

    vec2 newCoord_x = hor_mouth * newCoords.x;
    vec2 newCoord_y = ver_mouth * newCoords.y;

    if(abs(intensity_mouthmshape) > 0.001){
        out_texc += (newCoord_x + newCoord_y) * maskColor.a * intensity_mouthmshape * modify_intensity * scale_mouthmshape;
    }
    
    return out_texc;
}

vec2 MouthHeight(vec2 texc_warp) {
    vec2 out_texc = texc_warp;
    vec4 maskColor = texture2D(tex_mouthmask, texc + (out_texc - texc) / offset_wh.zw);
    maskColor = vec4(127.0 / 255.0,0.3,maskColor.a,maskColor.g * (1.0 - maskColor.b));
    vec2 newCoords = vec2(0.0);
    newCoords = (maskColor.rg - 127.0 / 255.0) / 25.0;

    vec2 off = abs(vec2(1.0) - texc_warp);
    vec2 off1 = abs(texc_warp);
    vec2 min_off = min(off, off1);
    vec2 modify_intensity = clamp(smoothstep(vec2(0.0), vec2(0.15, 0.1), min_off), vec2(0.55, 0.0), vec2(1.0, 1.0));

    vec2 newCoord_x = hor_mouth * newCoords.x;
    vec2 newCoord_y = ver_mouth * newCoords.y;

    if(abs(intensity_mouthheight) > 0.001){
        out_texc += (newCoord_x + newCoord_y) * maskColor.a * intensity_mouthheight * modify_intensity * scale_mouthheight;
    }
    
    return out_texc;
}

vec2 MouthSlant(vec2 texc_warp) {
    vec2 out_texc = texc_warp;
    vec4 maskColor = texture2D(tex_mouthmask, texc + (out_texc - texc) / offset_wh.zw);
    maskColor.ba = vec2(maskColor.a,maskColor.g * (1.0 - maskColor.b));
    vec2 newCoords = vec2(0.0);
    newCoords = (maskColor.rg - 127.0 / 255.0) / 25.0;

    vec2 off = abs(vec2(1.0) - texc_warp);
    vec2 off1 = abs(texc_warp);
    vec2 min_off = min(off, off1);
    vec2 modify_intensity = clamp(smoothstep(vec2(0.0), vec2(0.15, 0.1), min_off), vec2(0.55, 0.0), vec2(1.0, 1.0));

    vec2 newCoord_x = hor_mouth * newCoords.x;
    vec2 newCoord_y = ver_mouth * newCoords.y;

    if(abs(intensity_mouthslant) > 0.001){
        vec2 tex_o = (M_mouthslant * vec4(out_texc,0.0,1.0)).xy;
        out_texc += (tex_o - out_texc) * maskColor.a * modify_intensity * scale_mouthslant;
    }
    
    return out_texc;
}


void main(){
    vec2 out_texc = texc + texture2D(tex_uv_in, texc).rg;
    if (abs(intensity_mouthslant) > 0.001) {
        out_texc = MouthSlant(out_texc);
    }
    if (abs(intensity_mouthheight) > 0.001) {
        out_texc = MouthHeight(out_texc);
    }
    if (abs(intensity_mouthsize) > 0.001) {
        out_texc = MouthSize(out_texc);
    }
    if (abs(intensity_mouthwidth) > 0.001 || abs(left_intensity_mouthwidth) > 0.001 || abs(right_intensity_mouthwidth) > 0.001) {
        out_texc = MouthWidth(out_texc);
    }
    if (abs(intensity_mouththickness) > 0.001 || abs(up_intensity_mouththickness) > 0.001 || abs(down_intensity_mouththickness) > 0.001) {
        out_texc = MouthThickness(out_texc);
    }
    if (abs(intensity_mouthsmile) > 0.001) {
        out_texc = MouthSmile(out_texc);
    }
    if (abs(intensity_mouthmshape) > 0.001) {
        out_texc = MouthMshape(out_texc);
    }
    gl_FragColor = vec4(out_texc - texc, 0.0, 1.0);
}