varying vec2 texc;
uniform vec2 ver_ratio;
uniform vec2 hor_ratio;
uniform vec4 offset_wh;

uniform float intensity_faceshort;
uniform float scale_faceshort;
uniform sampler2D tex_faceshort;

uniform float intensity_forehead;
uniform float scale_forehead;
uniform sampler2D tex_forehead;

uniform float intensity_lowercourt;
uniform float scale_lowercourt;
uniform sampler2D tex_lowercourt;

uniform float intensity_middlecourt;
uniform float scale_middlecourt;
uniform sampler2D tex_courtmask;

uniform sampler2D tex_uv_in;

vec2 FaceShort(vec2 texc_warp) {
    vec2 out_texc = texc_warp;

    vec2 off = abs(vec2(1.0) - texc_warp);
    vec2 off1 = abs(texc_warp);
    vec2 min_off = min(off, off1);
    vec2 modify_intensity = clamp(smoothstep(vec2(0.0), vec2(0.15, 0.1), min_off), vec2(0.55, 0.0), vec2(1.0, 1.0));

    vec4 maskColor = texture2D(tex_faceshort, texc + (out_texc - texc) / offset_wh.zw);
    vec2 newCoords = (maskColor.rg - 127.0 / 255.0) / 25.0;
    vec2 newCoord_x = hor_ratio * newCoords.x;
    vec2 newCoord_y = ver_ratio * newCoords.y;

    out_texc += (newCoord_x + newCoord_y) * maskColor.a * intensity_faceshort * modify_intensity * scale_faceshort;
    
    return out_texc;
}

vec2 ForeHead(vec2 texc_warp) {
    vec2 out_texc = texc_warp;

    vec2 off = abs(vec2(1.0) - texc_warp);
    vec2 off1 = abs(texc_warp);
    vec2 min_off = min(off, off1);
    vec2 modify_intensity = clamp(smoothstep(vec2(0.0), vec2(0.15, 0.1), min_off), vec2(0.55, 0.0), vec2(1.0, 1.0));

    vec4 maskColor = texture2D(tex_forehead, texc + (out_texc - texc) / offset_wh.zw);
    maskColor.a = smoothstep(0.3,1.0,maskColor.a);
    vec2 newCoords = (maskColor.rg - 127.0 / 255.0) / 25.0;
    vec2 newCoord_x = hor_ratio * newCoords.x;
    vec2 newCoord_y = ver_ratio * newCoords.y;

    out_texc += (newCoord_x + newCoord_y) * maskColor.a * intensity_forehead * modify_intensity * scale_forehead;
    
    return out_texc;
}

vec2 LowerCourt(vec2 texc_warp) {
    vec2 out_texc = texc_warp;

    vec2 off = abs(vec2(1.0) - texc_warp);
    vec2 off1 = abs(texc_warp);
    vec2 min_off = min(off, off1);
    vec2 modify_intensity = clamp(smoothstep(vec2(0.0), vec2(0.15, 0.1), min_off), vec2(0.55, 0.0), vec2(1.0, 1.0));

    vec4 maskColor = texture2D(tex_lowercourt, texc + (out_texc - texc) / offset_wh.zw);
    maskColor.a = smoothstep(0.3,1.0,maskColor.a);
    vec2 newCoords = (maskColor.rg - 127.0 / 255.0) / 25.0;
    vec2 newCoord_x = hor_ratio * newCoords.x;
    vec2 newCoord_y = ver_ratio * newCoords.y;

    out_texc += (newCoord_x + newCoord_y) * maskColor.a * intensity_lowercourt * modify_intensity * scale_lowercourt;
    
    return out_texc;
}

vec2 MiddleCourt(vec2 texc_warp) {
    vec2 out_texc = texc_warp;
    vec4 maskColor = texture2D(tex_courtmask, texc + (out_texc - texc) / offset_wh.zw);
    maskColor = vec4(127.0 / 255.0,0.3,maskColor.a,maskColor.r);
    vec2 newCoords = vec2(0.0);
    newCoords = (maskColor.rg - 127.0 / 255.0) / 25.0;

    vec2 off = abs(vec2(1.0) - texc_warp);
    vec2 off1 = abs(texc_warp);
    vec2 min_off = min(off, off1);
    vec2 modify_intensity = clamp(smoothstep(vec2(0.0), vec2(0.15, 0.1), min_off), vec2(0.55, 0.0), vec2(1.0, 1.0));

    vec2 newCoord_x = hor_ratio * newCoords.x;
    vec2 newCoord_y = ver_ratio * newCoords.y;

    if(abs(intensity_middlecourt) > 0.001){
        out_texc += (newCoord_x + newCoord_y) * maskColor.a * intensity_middlecourt * modify_intensity * scale_middlecourt;
    }
    
    return out_texc;
}

void main(){
    vec2 out_texc = texc + texture2D(tex_uv_in, texc).rg;
    if (abs(intensity_faceshort) > 0.001) {
        out_texc = FaceShort(out_texc);
    }
    if (abs(intensity_forehead) > 0.001) {
        out_texc = ForeHead(out_texc);
    }
    if (abs(intensity_lowercourt) > 0.001) {
        out_texc = LowerCourt(out_texc);
    }
    if (abs(intensity_middlecourt) > 0.001) {
        out_texc = MiddleCourt(out_texc);
    }
    gl_FragColor = vec4(out_texc - texc, 0.0, 1.0);
}